
# q-expansion of newform 1728.2.a.d, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1728
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-3], [1], [-3], [4], [0], [2], [6], [-6], [-5], [-2], [-6], [-10], [-6], [-9], [12], [-8], [14], [0], [-7], [-8], [-3], [-18], [-1], [3], [4], [9], [-2], [-6], [7], [-15], [6], [-4], [-3], [-17], [4], [20], [6], [-15], [-9], [16], [12], [5], [-9], [7], [-22], [-8], [-12], [-14], [18], [-30], [-10], [0], [12], [30], [18], [25], [-8], [24], [14], [6], [-16], [6], [-19], [3], [-10], [-22], [3], [10], [6], [-18], [-17], [-32], [20], [24], [21], [-20], [12], [23], [12], [-8], [-18], [29], [19], [-12], [-18], [-1], [21], [13], [-27], [-6], [16], [39], [14], [18], [15], [36], [-16], [-20], [8], [-27], [3], [-12], [-4], [38], [-3], [18], [42], [35], [-32], [34], [-42], [-28], [25], [42], [-4], [0], [-39], [-21], [-14], [-19], [-42], [-36], [44], [-9], [-44], [-36], [1], [22], [-16], [-12], [-41], [-2], [48], [-31], [18], [32], [-39], [18], [2], [-6], [31], [0], [-2], [-24], [10], [-48], [-40], [36], [22], [-36], [2], [-12], [26], [-6], [-29], [-12], [11], [33], [51], [-36], [-23], [-27], [-42], [-6], [-47], [28], [11], [6], [15], [58], [-30], [26], [31], [-6], [26], [9], [-47], [64], [55], [33], [-14], [36], [-12], [-15], [-38], [8], [59], [54], [2], [-57], [2], [57], [-36], [12], [23], [-62], [-12], [-36], [-39], [55], [-32], [-19], [-9], [18], [-20], [21], [-30], [-40], [-7], [42], [-35], [-21], [24], [-55], [-32], [42], [-18], [6], [-14], [4], [-24], [-20], [57], [-8], [24], [54], [-8], [-12], [-14], [56], [40], [12], [-10], [-60], [14], [-18], [39], [-36], [60], [-34], [49], [16], [-24], [12], [43], [57], [14], [-36], [76], [72], [-30], [17], [75], [-63], [52], [8], [-9], [65], [1], [-21], [58], [16], [12], [-28], [18], [6], [-4], [6], [-32], [-16], [53], [-59], [-31], [79], [-51], [-8], [47], [-12], [60], [-77], [30], [-26], [-22], [-18], [-61], [-3], [64], [0], [-42], [45], [-60], [-69], [10], [-66], [-17], [39], [-63], [26], [-22], [-27], [25], [-51], [-22], [47], [60], [-86], [24], [16], [84], [-9], [-30], [68], [72], [-70], [-39], [-12], [-46], [30], [-28], [-67], [27], [-11], [24], [47], [74], [-64], [48], [-63], [46], [6], [-17], [24], [-16], [9], [-2], [-78], [41], [61], [46], [-48], [-93], [-8], [-33], [-36], [-74], [-82], [-42], [-54], [-40], [11], [-66], [-89], [-14], [18], [-30], [63], [-78], [-42], [64], [24], [-36], [15], [74], [62], [-78], [-29], [-7], [69], [20], [-30], [-33], [31], [52], [-96], [-54], [11], [-54], [-82], [21], [30], [88], [-48], [-4], [-54], [28], [4], [74], [-12], [17], [30], [-36], [26], [12], [14], [16], [-30], [-22], [33], [4], [-90], [40], [-24], [-45], [43], [-26], [-12], [-4], [-21], [-1], [-87], [81], [44], [41], [63], [-18], [-8], [-54], [6], [66], [-2], [-60], [57], [-43], [-57], [-39], [54], [56], [12]]
